@@ -36,10 +36,11 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
-#include "magick/cache.h"
+#include "magick/pixel_cache.h"
 #include "magick/composite.h"
 #include "magick/log.h"
 #include "magick/magick.h"
+#include "magick/monitor.h"
 #include "magick/quantize.h"
 #include "magick/utility.h"
 
@@ -235,13 +236,13 @@ static CompositeOperator GIMPBlendModeToCompositeOperator( unsigned int blendMod
     case GIMP_HUE_MODE:      return( HueCompositeOp );
     case GIMP_SATURATION_MODE:  return( SaturateCompositeOp );
     case GIMP_COLOR_MODE:    return( ColorizeCompositeOp );
+    case GIMP_DIVIDE_MODE:    return( DivideCompositeOp );
       /* these are the ones we don't support...yet */
     case GIMP_DODGE_MODE:    return( OverCompositeOp );
     case GIMP_BURN_MODE:    return( OverCompositeOp );
     case GIMP_HARDLIGHT_MODE:  return( OverCompositeOp );
     case GIMP_BEHIND_MODE:    return( OverCompositeOp );
     case GIMP_VALUE_MODE:    return( OverCompositeOp );
-    case GIMP_DIVIDE_MODE:    return( OverCompositeOp );
     default:
       return( OverCompositeOp );
     }
@@ -614,6 +615,7 @@ static MagickPassFail load_tile_rle (Image* image,
   return MagickFail;
 }
 
+
 static MagickPassFail load_level (Image* image,
                                   XCFDocInfo* inDocInfo,
                                   XCFLayerInfo*
@@ -683,7 +685,7 @@ static MagickPassFail load_level (Image* image,
 
   /*
     Initialise the reference for the in-memory tile-compression
-   */
+  */
   ntile_rows = (inDocInfo->height + TILE_HEIGHT - 1) / TILE_HEIGHT;
   ntile_cols = (inDocInfo->width  + TILE_WIDTH  - 1) / TILE_WIDTH;
   ntiles = ntile_rows * ntile_cols;
@@ -701,14 +703,14 @@ static MagickPassFail load_level (Image* image,
         {
           if (image->logging)
             (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                            "Tile: offset %ld!", (long) offset);
+                                  "Tile: offset %ld!", (long) offset);
           ThrowBinaryException(CorruptImageError,NotEnoughTiles,image->filename);
         }
 
       /*
         save the current position as it is where the next tile offset
         is stored.
-       */
+      */
       saved_pos = TellBlob(image);
 
       /*
@@ -816,9 +818,60 @@ static MagickPassFail load_level (Image* image,
 
       if (MagickPass == status)
         {
-          /* composite the tile onto the layer's image, and then destroy it */
-          (void) CompositeImage(inLayerInfo->image,CopyCompositeOp,tile_image,
-                                destLeft * TILE_WIDTH,destTop*TILE_HEIGHT);
+          /*
+            Composite the tile onto the layer's image, and then
+            destroy it.  We temporarily disable the progress monitor
+            so that the user does not see composition of individual
+            tiles.
+          */
+#if 0
+          const PixelPacket
+            *p;
+
+          PixelPacket
+            *q;
+
+          long
+            canvas_x,
+            canvas_y,
+            y;
+
+          unsigned long
+            tile_width;
+
+          canvas_x=destLeft*TILE_WIDTH;
+          tile_width=tile_image->columns;
+          for (y=0; y < (long) tile_image->columns; y++)
+            {
+              canvas_y=destTop*TILE_HEIGHT+y;
+              p=AcquireImagePixels(tile_image,0,y,tile_image->columns,1,
+                                   &inLayerInfo->image->exception);
+              q=GetImagePixels(inLayerInfo->image,canvas_x,canvas_y,
+                               tile_image->columns,1);
+              if ((p != (const PixelPacket *) NULL) && (q != (PixelPacket *) NULL))
+                (void) memcpy(q,p,tile_image->columns*sizeof(PixelPacket));
+              else
+                printf("null pointer canvas: %lux%lu tile: %lux%lu+%ld+%ld !\n",
+                       inLayerInfo->image->columns,inLayerInfo->image->rows,
+                       tile_width,1LU,canvas_x,canvas_y);
+            }
+#else
+          MonitorHandler
+            handler;
+
+          long
+            canvas_x,
+            canvas_y;
+
+          canvas_x=destLeft*TILE_WIDTH;
+          canvas_y=destTop*TILE_HEIGHT;
+          handler=SetMonitorHandler((MonitorHandler) NULL);
+          (void) CompositeImageRegion(CopyCompositeOp,NULL,tile_image->columns,
+                                      tile_image->rows,tile_image,0,0,
+                                      inLayerInfo->image,canvas_x,
+                                      canvas_y,&inLayerInfo->image->exception);
+          (void) SetMonitorHandler(handler);
+#endif
         }
       DestroyImage(tile_image);
 
@@ -840,11 +893,25 @@ static MagickPassFail load_level (Image* image,
 
       /* read in the offset of the next tile */
       offset = ReadBlobMSBLong(image);
+      if (offset != 0)
+        if (!MagickMonitorFormatted(offset,inDocInfo->file_size,
+                                    &image->exception,LoadImageText,
+                                    image->filename,
+				    image->columns,image->rows))
+          break;
     }
 
-
   if (offset != 0)
-    ThrowBinaryException(CorruptImageError,CorruptImage,image->filename);
+    {
+      ThrowBinaryException(CorruptImageError,CorruptImage,image->filename);
+    }
+  else
+    {
+      (void) MagickMonitorFormatted(inDocInfo->file_size,
+                                    inDocInfo->file_size+1,&image->exception,
+                                    LoadImageText,image->filename,
+				    image->columns,image->rows);
+    }
 
   return MagickPass;
 }
@@ -1206,7 +1273,11 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     {
       ThrowReaderException(CoderError,ColormapTypeNotSupported,image);
     }
-  (void) SetImage(image,OpaqueOpacity);  /* until we know otherwise...*/
+  /*
+    SetImage can be very expensive and it is not clear that this one is
+    actually needed so comment it out for now.
+   */
+  /* (void) SetImage(image,OpaqueOpacity); */  /* until we know otherwise...*/
   image->matte=True;  /* XCF always has a matte! */
 
   /* read properties */
@@ -1423,8 +1494,8 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
 
       /* allocate our array of layer info blocks */
-      layer_info=MagickAllocateMemory(XCFLayerInfo *,
-                                     number_layers*
+      layer_info=MagickAllocateArray(XCFLayerInfo *,
+                                     number_layers,
                                      sizeof(XCFLayerInfo));
       if (layer_info == (XCFLayerInfo *) NULL)
         ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
@@ -1609,8 +1680,8 @@ ModuleExport void RegisterXCFImage(void)
   entry=SetMagickInfo("XCF");
   entry->decoder=(DecoderHandler) ReadXCFImage;
   entry->magick=(MagickHandler) IsXCF;
-  entry->description=AcquireString("GIMP image");
-  entry->module=AcquireString("XCF");
+  entry->description="GIMP image";
+  entry->module="XCF";
   entry->seekable_stream=True;
   (void) RegisterMagickInfo(entry);
 }
